Package: generics
Title: Common S3 Generics not Provided by Base R Methods Related to
        Model Fitting
Version: 0.1.4
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Max", "Kuhn", , "max@posit.co", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "https://ror.org/03wc8by49"))
  )
Description: In order to reduce potential package dependencies and
    conflicts, generics provides a number of commonly used S3 generics.
License: MIT + file LICENSE
URL: https://generics.r-lib.org, https://github.com/r-lib/generics
BugReports: https://github.com/r-lib/generics/issues
Depends: R (>= 3.6)
Imports: methods
Suggests: covr, pkgload, testthat (>= 3.0.0), tibble, withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-05-09 18:17:41 UTC; hadleywickham
Author: Hadley Wickham [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Max Kuhn [aut],
  Davis Vaughan [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-05-09 23:50:06 UTC
Built: R 4.4.0; ; 2026-07-17 18:14:26 UTC; unix

Package: testSingleMethod
Title: Tools to make developing R code easier
License: GPL-2
Description:
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate: a.r
RoxygenNote: 6.1.0

Package: generics
Title: Common S3 Generics not Provided by Base R Methods Related to
        Model Fitting
Version: 0.1.4
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Max", "Kuhn", , "max@posit.co", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "https://ror.org/03wc8by49"))
  )
Description: In order to reduce potential package dependencies and
    conflicts, generics provides a number of commonly used S3 generics.
License: MIT + file LICENSE
URL: https://generics.r-lib.org, https://github.com/r-lib/generics
BugReports: https://github.com/r-lib/generics/issues
Depends: R (>= 3.6)
Imports: methods
Suggests: covr, pkgload, testthat (>= 3.0.0), tibble, withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-05-09 18:17:41 UTC; hadleywickham
Author: Hadley Wickham [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Max Kuhn [aut],
  Davis Vaughan [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-05-09 23:50:06 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.character,dev_topic)
S3method(as.difftime,default)
S3method(as.factor,default)
S3method(as.ordered,default)
S3method(intersect,default)
S3method(is.element,default)
S3method(setdiff,default)
S3method(setequal,default)
S3method(union,default)
export(accuracy)
export(as.difftime)
export(as.factor)
export(as.ordered)
export(augment)
export(calculate)
export(compile)
export(components)
export(equation)
export(estfun)
export(evaluate)
export(explain)
export(explore)
export(fit)
export(fit_xy)
export(forecast)
export(generate)
export(glance)
export(hypothesize)
export(interpolate)
export(intersect)
export(is.element)
export(learn)
export(min_grid)
export(prune)
export(rank_results)
export(refit)
export(required_pkgs)
export(setdiff)
export(setequal)
export(specify)
export(tidy)
export(train)
export(tunable)
export(tune_args)
export(union)
export(var_imp)
export(varying_args)
export(visualize)

# generics

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/generics/workflows/R-CMD-check/badge.svg)](https://github.com/r-lib/generics/actions)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/generics/branch/master/graph/badge.svg)](https://codecov.io/gh/r-lib/generics?branch=master)
<!-- badges: end -->

`generics` is designed to help package authors reduce dependencies by
providing a set of generic methods that can be imported. For example, if
a package author wanted to include a `tidy` method for their object,
they would have to import the `broom` package to do so. This would work
but would potentially increase the number of package dependencies
required to install and/or test the package.

## Installation

To install `generics` from CRAN, use:

``` r
install.packages("generics")
```

To install the development version, use:

``` r
install.packages("devtools")
devtools::install_github("r-lib/generics")
```

## Usage

`generics` is a simple, lightweight package that contains S3 generics to
be used by other packages. Some examples are:

``` r
library(generics)
#> 
#> Attaching package: 'generics'
#> The following objects are masked from 'package:base':
#> 
#>     as.difftime, as.factor, as.ordered, intersect, is.element, setdiff,
#>     setequal, union

fit
#> function(object, ...) {
#>   UseMethod("fit")
#> }
#> <bytecode: 0x7fca8cc62a30>
#> <environment: namespace:generics>

tidy
#> function(x, ...) {
#>   UseMethod("tidy")
#> }
#> <bytecode: 0x7fca6c986cf8>
#> <environment: namespace:generics>
```

To use `generics` with your package, we recommend that you import and
re-export the generic(s) of interest. For example, if you want to
provide a method for the S3 `explain()` method, you’d using the
following `roxygen2` code:

``` r
#' @importFrom generics explain
#' @export
generics::explain
```

As an example, the [recipes](https://github.com/tidymodels/recipes)
package defines a number of `tidy()` S3 methods by importing this
package (whereas it previously depended on `broom`).

## Documentation

When searching for help on a method that is exported from `generics` by
one or more packages, using `?method` will show entries for all exported
methods. If the version from `generics` is selected, the Methods section
dynamically lists all specific methods exported by any loaded packages.

# generics

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/generics/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/generics/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/generics/branch/main/graph/badge.svg)](https://app.codecov.io/gh/r-lib/generics?branch=main)
<!-- badges: end -->

`generics` is designed to help package authors reduce dependencies by
providing a set of generic methods that can be imported. For example, if
a package author wanted to include a `tidy` method for their object,
they would have to import the `broom` package to do so. This would work
but would potentially increase the number of package dependencies
required to install and/or test the package.

## Installation

To install `generics` from CRAN, use:

``` r
install.packages("generics")
```

To install the development version, use:

``` r
# install.packages("pak")
pak::pak("r-lib/generics")
```

## Usage

`generics` is a simple, lightweight package that contains S3 generics to
be used by other packages. Some examples are:

``` r
library(generics)
#> 
#> Attaching package: 'generics'
#> The following objects are masked from 'package:base':
#> 
#>     as.difftime, as.factor, as.ordered, intersect, is.element, setdiff,
#>     setequal, union

fit
#> function(object, ...) {
#>   UseMethod("fit")
#> }
#> <bytecode: 0x107df2b38>
#> <environment: namespace:generics>

tidy
#> function(x, ...) {
#>   UseMethod("tidy")
#> }
#> <bytecode: 0x107e583a8>
#> <environment: namespace:generics>
```

To use `generics` with your package, we recommend that you import and
re-export the generic(s) of interest. For example, if you want to
provide a method for the S3 `explain()` method, you’d using the
following `roxygen2` code:

``` r
#' @importFrom generics explain
#' @export
generics::explain
```

As an example, the [recipes](https://github.com/tidymodels/recipes)
package defines a number of `tidy()` S3 methods by importing this
package (whereas it previously depended on `broom`).

## Documentation

When searching for help on a method that is exported from `generics` by
one or more packages, using `?method` will show entries for all exported
methods. If the version from `generics` is selected, the Methods section
dynamically lists all specific methods exported by any loaded packages.

#' Generic test for multiple methods
#'
#'
#' @param x,y A parameter
#'
#' @rdname multi-method
#'
#' @export
multi_method <- function(x, y) {
  UseMethod("multi_method")
}

#' Test default
#'
#' @param x,y A parameter
#'
#' @rdname multi-method-2
#'
#' @export
multi_method.default <- function(x, y) {
  x
}

#' Test data frame
#'
#' @param x,y A parameter
#'
#' @rdname multi-method-3
#'
#' @export
multi_method.data.frame <- function(x, y) {
  x
}

#' An s4 generic
#'
#' @export
#' @aliases multi_method,ANY,ANY-method
setGeneric("multi_method", function(x, y) {
  x
})

#' @rdname multi_method
#' @export
setMethod("multi_method", signature(x = "numeric"), function(x, y) {
  x
})

#' @rdname multi_method
#' @export
setMethod("multi_method", signature(x = "numeric", y = "integer"), function(x, y) {
  x
})

#' Generic test
#'
#'
#' @param x A parameter
#'
#' @rdname single-method
#'
#' @export
single_method <- function(x) {
  UseMethod("single_method")
}

#' Test default
#'
#' @param x A parameter
#'
#' @rdname single-method-2
#'
#' @export
single_method.default <- function(x) {
  x
}

#' Generic test for multiple methods on the same rd file
#'
#' Should be comma separated
#'
#' @param x A parameter
#'
#' @rdname same_rd_name
#'
same_rd_name <- function(x) {
  UseMethod("same_rd_name")
}

#' Test default
#'
#' @param x A parameter
#'
#' @rdname same_rd_name-2
same_rd_name.default <- function(x) {
  x
}

#' Test data frame
#'
#' @param x A parameter
#'
#' @rdname same_rd_name-2
same_rd_name.data.frame <- function(x) {
  x
}

#' Extracting the estimating functions of a fitted model.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("estfun")}
#'
#' @param x A fitted model object.
#' @param ... Other arguments passed to methods
#' @export
estfun <- function(x, ...) {
  UseMethod("estfun")
}

#' Prune or reduce an object
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("prune")}
#'
#' @param tree A fitted model object.
#' @param ... Other arguments passed to methods
#' @export
prune <- function(tree, ...) {
  UseMethod("prune")
}

#' Evaluate an object.
#'
#' @param x An object. See the individual method for specifics.
#' @param ... other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("evaluate")}
#'
#' @inheritParams augment
#' @export
evaluate <- function(x, ...) {
  UseMethod("evaluate")
}

#' generics: common S3 generics
#'
#' These are generic functions that can be used to minimize
#'  package dependencies when multiple packages have the same
#'  method.
#'
#' An attempt was made to use generic class signatures that were
#'  consistent with existing code. For example, \pkg{dplyr} and
#'  \pkg{lime} both have `explain()` methods that do very different
#'  things but both take `x` as their main object. Even though `x`
#'  would be different in those cases, this package provides access
#'  to the S3 generic so that other packages that may want to create
#'  `explain()` methods for their objects can do so without loading
#'  either of the other packages.
#'
#' For example, if a new `tidy()` method is being developed for a
#'  package, this lightweight package can be the required dependency
#'  to have access to the generic method (instead of depending on
#'  \pkg{broom} and installing its dependencies).
#'
#' @keywords internal
"_PACKAGE"

#' Determine the minimum set of model fits
#'
#' `min_grid()` determines exactly what models should be fit in order to
#'  evaluate the entire set of tuning parameter combinations. This is for
#'  internal use only and the API may change in the near future.
#' @param x A model specification.
#' @param grid A tibble with tuning parameter combinations.
#' @param ... Not currently used.
#' @return A tibble with the minimum tuning parameters to fit and an additional
#' list column with the parameter combinations used for prediction.
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("min_grid")}
#' @export
min_grid <- function(x, grid, ...) {
  UseMethod("min_grid")
}

#' Visualize a data set or object.
#'
#' @param x A data frame or other object.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("visualize")}
#'
#' @export
visualize <- function(x, ...) {
  UseMethod("visualize")
}

#' Model equations
#'
#' Display the mathematical representation of a fitted model.
#'
#' @param object A fitted model object.
#' @param ... Other arguments passed to methods
#'
#' @return Markup output suitable for rendering the equation.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("equation")}
#'
#' @export
equation <- function(object, ...) {
  UseMethod("equation")
}

#' Determine packages required by objects
#'
#' @param x An object.
#' @param ... Other arguments passed to methods
#' @return A character string of packages that are required.
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("required_pkgs")}
#' @export
required_pkgs <- function(x, ...) {
  UseMethod("required_pkgs")
}

#' Configure an object
#'
#' Finalizes or completes an object.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("compile")}
#'
#' @param object An object. See the individual method for specifics.
#' @param ... Other arguments passed to methods
#' @export
compile <- function(object, ...) {
  UseMethod("compile")
}

#' Augment data with information from an object
#'
#' @param x Model object or other R object with information to append to
#'   observations.
#' @param ... Addition arguments to `augment` method.
#' @return A [tibble::tibble()] with information about data points.
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("augment")}
#'
#' @export
#'
augment <- function(x, ...) {
  UseMethod("augment")
}

#' Glance at an object
#'
#' Construct a single row summary "glance" of a model, fit, or other
#' object
#'
#' glance methods always return either a one-row data frame (except on
#'  `NULL`, which returns an empty data frame)
#'
#' @param x model or other R object to convert to single-row data frame
#' @param ... other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("glance")}
#'
#' @export
glance <- function(x, ...) {
  UseMethod("glance")
}

#' Compute relative rankings of a collection of objects
#'
#' `rank_results()` computes relative ranks of a collection of objects and
#' returns a summary of the results.
#'
#' @param x A collection of objects
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("rank_results")}
#'
#' @export
rank_results <- function(x, ...) {
  UseMethod("rank_results")
}

#' Create an interactive visualization appropriate to a particular object type
#'
#' `explore()` invokes a function that starts an interactive, pre-defined widget
#' (e.g. `plotly` visualization, `shiny` app, etc.) to investigate the results.
#'
#' @param x A object
#' @param ... Other arguments passed to methods
#'
#' @return `NULL` (invisibly) or some other data type (e.g. tibble) depending on
#' the application.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("explore")}
#'
#' @export
explore <- function(x, ...) {
  UseMethod("explore")
}

#' Refitting models
#'
#' @param object A fitted model object.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("refit")}
#'
#' @export
refit <- function(object, ...) {
  UseMethod("refit")
}

#' Interpolate missing values
#'
#' Interpolates missing values provided in the training dataset using the
#' fitted model.
#'
#' @param object A fitted model object
#' @param ... Other arguments passed to methods
#'
#' @return A dataset ([tibble::tibble()] or similar) of the same structure as
#' the input dataset with missing values from the response variable replaced
#' with interpolated values.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("interpolate")}
#'
#' @export
interpolate <- function(object, ...) {
  UseMethod("interpolate")
}

#' Turn an object into a tidy tibble
#'
#' @param x An object to be converted into a tidy [tibble::tibble()].
#' @param ... Additional arguments to tidying method.
#' @return A [tibble::tibble()] with information about model components.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("tidy")}
#'
#' @export
tidy <- function(x, ...) {
  UseMethod("tidy")
}

#' Specify variables or other quantities.
#'
#' @param x An object.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("specify")}
#'
#' @export
specify <- function(x, ...) {
  UseMethod("specify")
}

#' Factor coercion
#'
#' Coercion functions for creating factors from other existing objects.
#'
#' These functions override non-generic factor coercion functions provided
#' in base so that packages can provide methods for different data types. The
#' default methods call the base versions.
#'
#' @section Methods:
#'
#' \subsection{`as.factor()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("as.factor")}
#' }
#'
#' \subsection{`as.ordered()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("as.ordered")}
#' }
#'
#'
#' @param x A vector of data.
#' @param ... Other arguments passed on to methods.
#' @return For `as.factor()`, a factor. For `as.ordered()`,
#' an ordered factor.
#'
#' @name coercion-factor
#'
#' @examples
#' as.factor(letters[1:5])
#' as.ordered(letters[1:5])
#'
NULL

#' @rdname coercion-factor
#' @export
as.factor <- function(x, ...) UseMethod("as.factor")

#' @rdname coercion-factor
#' @export
as.ordered <- function(x, ...) UseMethod("as.ordered")


#' @export
as.factor.default <- function(x, ...) base::as.factor(x)

#' @export
as.ordered.default <- function(x, ...) base::as.ordered(x)


#' Time difference coercion
#'
#' Coercion functions for creating `difftime` objects from other
#' existing objects.
#'
#' This function overrides the non-generic `as.difftime()` function
#' provided in base so that packages can provide methods for different data
#' types. The default method call the base version.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("as.difftime")}
#'
#' @param tim A vector specifying a time interval.
#' @param format A single character specifying the format of `tim` when it is
#' a character. The default is a locale-specific time format.
#' @param units A single character specifying units in which the results are
#' desired. Required if `tim` is a numeric.
#' @param ... Other arguments passed on to methods.
#' @return A `difftime` object with an attribute indicating the units.
#'
#' @name coercion-time-difference
#'
#' @examples
#' as.difftime(1:5, units = "secs")
#'
#' as.difftime(c("01:55:22", "01:55:25"))
#'
#' as.difftime("01", format = "%H")
#' as.difftime("01", format = "%H", units = "secs")
#'
NULL

#' @rdname coercion-time-difference
#' @export
as.difftime <- function(tim, ...) UseMethod("as.difftime")


#' @rdname coercion-time-difference
#' @export
as.difftime.default <- function(tim, format = "%X", units = "auto", ...)
  base::as.difftime(tim = tim, format = format, units = units, ...)

#' Declare tunable parameters
#'
#' Returns information on potential hyper-parameters that can be optimized.
#'
#' @param x An object, such as a recipe, recipe step, workflow, or model
#' specification.
#' @param ... Other arguments passed to methods
#'
#'@return A tibble with a column for the parameter `name`, information on the
#'  _default_ method for generating a corresponding parameter object, the
#'  `source` of the parameter (e.g. "recipe", etc.), and the `component` within
#'  the source. For the `component` column, a little more specificity is given
#'  about the location of the parameter (e.g. "step_normalize" or recipes or
#'  "boost_tree" for models). The `component_id` column contains the unique step
#'  `id` field or, for models, a logical for whether the model specification
#'  argument was a main parameter or one associated with the engine.
#' @details
#' For a model specification, an engine must be chosen.
#'
#' If the object has no tunable parameters, a tibble with no rows is returned.
#'
#' The information about the default parameter object takes the form of a
#' named list with an element for the function call and an optional element for
#' the source of the function (e.g. the `dials` package). For model
#' specifications, If the parameter is unknown to the underlying `tunable`
#' method, a `NULL` is returned.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("tunable")}
#'
#' @export
tunable <- function(x, ...) {
  UseMethod("tunable")
}

#' Find any arguments that are not fully specified.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("varying_args")}
#'
#' @inheritParams compile
#' @export
varying_args <- function(object, ...) {
  UseMethod("varying_args")
}

#' Estimate model parameters.
#'
#' Estimates parameters for a given model from a set of data.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("fit")}
#'
#' @inheritParams compile
#' @export
fit <- function(object, ...) {
  UseMethod("fit")
}

#' Extract components
#'
#' `components` can be used to extract elements from an object.
#'
#' For example, decomposition methods and some modelling techniques can be used
#' to decompose a dataset into components of interest. This function is used
#' to extract these components in a tidy data format.
#'
#' @param object A data separable object.
#' @param ... Other arguments passed to methods
#'
#' @return A dataset ([tibble::tibble()] or similar) containing components from
#' the object.
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("components")}
#'
#' @export
components <- function(object, ...) {
  UseMethod("components")
}

#' Determine arguments tagged for tuning
#'
#' `tune_args()` takes an object such as a model specification or a recipe and
#' returns a tibble of information on all possible tunable arguments and
#' whether or not they are actually tunable.
#'
#' @param object A `model_spec`, `recipe`, `workflow`, or other object.
#' @param ... Other arguments passed to methods.
#'
#' @details
#' The `source` column is determined differently for a
#' `model_spec` or a `recipe` (with additional detail on the type).
#'
#' The `id` field has any identifier that was passed from `tune::tune()` (e.g.
#' `tune("some note")`). If no additional detail was used in that function,
#' the `id` field reverts to the name of the parameters.
#'
#' @return A tibble with columns for the parameter name (`name`), whether it
#' contains _any_ tunable value (`tune`), the `id` for the parameter (`id`),
#' and the information on where the parameter was located (`source`).
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("tune_args")}
#'
#' @export
tune_args <- function(object, ...) {
  UseMethod("tune_args")
}

#' Estimate model parameters.
#'
#' Estimates parameters for a given model from a set of data.
#'
#' @param x An object. See the individual method for specifics.
#' @param ... other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("learn")}
#'
#' @inheritParams compile
#' @export
learn <- function(x, ...) {
  UseMethod("learn")
}

#' Calculation of variable importance
#'
#' A generic method for calculating variable importance for model objects.
#'
#' @param object A fitted model object.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("var_imp")}
#'
#' @inheritParams compile
#' @export
var_imp <- function(object, ...) {
  UseMethod("var_imp")
}

# Modified from sloop::methods_generic
methods_find <- function(x) {
  info <- attr(utils::methods(x), "info")

  if (nrow(info) == 0) {
    info$topic <- character()
    return(info)
  }

  info$method <- rownames(info)
  rownames(info) <- NULL

  if (getRversion() < "3.2") {
    info$isS4 <- grepl("-method$", info$method)
  }

  # Simply class and source
  generic_esc <- gsub("\\.", "\\\\.", x)
  info$class <- gsub(paste0("^", generic_esc, "[.,]"), "", info$method)
  info$class <- gsub("-method$", "", info$class)
  info$source <- gsub(paste0(" for ", generic_esc), "", info$from)

  # Find package
  info$package <- lookup_package(x, info$class, info$isS4)

  # Find help topic
  path <- help_path(info$method, info$package)
  pieces <- strsplit(path, "/")
  info$topic <- vapply(pieces, last, character(1))

  info[c("method", "class", "package", "topic", "visible", "source")]
}

methods_rd <- function(x) {
  methods <- methods_find(x)
  methods <- methods[!is.na(methods$topic), , drop = FALSE]

  if (nrow(methods) == 0) {
    return("No methods found in currently loaded packages.")
  }

  methods_by_package <- split(methods, methods$package)

  topics_by_package <- lapply(methods_by_package, function(x) {
    split(x, paste(x$topic, x$package, sep = "."))
  })

  make_bullets <- function(topics) {
    bullet_vec <- vapply(
      X = topics,
      FUN = function(x) {
        link <- paste0(
          "\\code{",
          "\\link[", x$package[[1]], "]",
          "{", x$topic[[1]], "}",
          "}"
        )
        classes <- paste0("\\code{", x$class, "}", collapse = ", ")
        paste0("\\item ", link, ": ", classes)
      },
      FUN.VALUE = character(1),
      USE.NAMES = FALSE
    )

    paste0(bullet_vec, collapse = "\n")
  }

  make_header <- function(pkg) {
    paste0("\\code{", pkg, "}")
  }

  bullets <- lapply(topics_by_package, make_bullets)
  headers <- lapply(names(topics_by_package), make_header)

  help_msg <- paste0(
    "See the following help topics for more details about individual methods:\n"
  )

  paste0(
    c(help_msg,
      paste(
        headers,
        "\\itemize{",
        bullets,
        "}",
        sep = "\n"
      )
    ),
    collapse = "\n"
  )

}

last <- function(x, n = 0) {
  if (length(x) <= n) {
    x[NA_integer_]
  } else {
    x[[length(x) - n]]
  }
}

help_path <- function(x, package) {

  help <- mapply(locate_help_doc, x, package, SIMPLIFY = FALSE)

  vapply(help,
    function(x) if (length(x) == 0) NA_character_ else as.character(x),
    FUN.VALUE = character(1)
  )
}

locate_help_doc <- function(x, package) {
  help <- if (requireNamespace("pkgload", quietly = TRUE)) {
    shim_help <- get("shim_help", asNamespace("pkgload"))
    function(x, package = NULL) {
      tryCatch(
        expr = shim_help(x, (package)),
        error = function(e) character()
      )
    }
  } else {
    utils::help
  }

  if (is.na(package)) {
    help(x)
  } else {
    help(x, (package))
  }
}

as.character.dev_topic <- function(x, ...) {
  sub("[.]Rd$", "", x$path)
}

lookup_package <- function(generic, class, is_s4) {

  lookup_single_package <- function(generic, class, is_s4) {

    if (is_s4) {

      class <- strsplit(class, ",")[[1]]
      fn <- methods::getMethod(generic, class, optional = TRUE)

    } else {

      fn <- utils::getS3method(generic, class, optional = TRUE)

    }

    # Not found
    if (is.null(fn)) {
      return(NA_character_)
    }

    pkg <- utils::packageName(environment(fn))

    # Function method found, but in a non-package environment
    if (is.null(pkg)) {
      return(NA_character_)
    }

    pkg
  }

  pkgs <- mapply(lookup_single_package, generic, class, is_s4, SIMPLIFY = FALSE)
  as.vector(pkgs, "character")
}

test_that("generics methods can be reexported and extended", {
  local_load_all("testGenericsExtension")
  expect_snapshot(methods_rd("tidy"))
})

test_that("multiple packages have multiple headers", {
  local_load_all("testMultiMethod")
  local_load_all("testMultiPackage")
  expect_snapshot(methods_rd("multi_method"))
})

test_that("S4 bullets print with no issues", {
  local_load_all("testS4Docs")

  expect_snapshot(methods_rd("multi_method"))
})

test_that("S4 and S3 packages can intermingle", {
  local_load_all("testS4Docs")
  local_load_all("testMultiMethod")

  expect_snapshot(methods_rd("multi_method"))
})

test_that("multiple methods but same rdname are comma separated", {
  local_load_all("testSameRd")
  expect_snapshot(methods_rd("same_rd_name"))
})

test_that("single method is correctly itemized", {
  local_load_all("testSingleMethod")
  expect_snapshot(methods_rd("single_method"))
})

test_that("multiple methods are correctly itemized", {
  local_load_all("testMultiMethod")
  expect_snapshot(methods_rd("multi_method"))
})

test_that("no methods case returns default output", {
  expect_equal(
    methods_rd("methods_rd"),
    "No methods found in currently loaded packages."
  )
})

test_that("nonexistant generic is an error", {
  expect_error(methods_rd("xyz"))
})

context("test-same-rd")

library(pkgload)
library(testthat)

# Setup ------------------------------------------------------------------------

# Load package and register methods
load_all("testSameRd")

# Unload when finished testing
teardown({
  unload("testSameRd")
})

# Testing ----------------------------------------------------------------------

test_that("multiple methods but same rdname are comma separated", {
  expect_known_cat(
    methods_rd("same_rd_name"),
    "testSameRd/test-1.txt"
  )
})

context("test-s4")

library(pkgload)
library(testthat)

# Setup ------------------------------------------------------------------------

# Load package and register methods
load_all("testS4Docs")

# Unload when finished testing
teardown({
  unload("testS4Docs")
})

# Testing ----------------------------------------------------------------------

test_that("S4 bullets print with no issues", {

  skip_if_below_r_version("3.2")

  expect_known_cat(
    methods_rd("multi_method"),
    "testS4Docs/test-1.txt"
  )
})


# Load package and register methods
load_all("testMultiMethod")

# Unload when finished testing
teardown({
  unload("testMultiMethod")
})

test_that("S4 and S3 packages can intermingle", {

  skip_if_below_r_version("3.2")

  expect_known_cat(
    methods_rd("multi_method"),
    "testS4Docs/test-2.txt"
  )
})

context("test-single-package")

library(pkgload)
library(testthat)

# Setup ------------------------------------------------------------------------

# Load package and register methods
load_all("testSingleMethod")
load_all("testMultiMethod")

# Unload when finished testing
teardown({
  unload("testSingleMethod")
  unload("testMultiMethod")
})

# Testing ----------------------------------------------------------------------

test_that("single method is correctly itemized", {
  expect_known_cat(
    methods_rd("single_method"),
    "testSingleMethod/test-1.txt"
  )
})

test_that("multiple methods are correctly itemized", {
  expect_known_cat(
    methods_rd("multi_method"),
    "testMultiMethod/test-1.txt"
  )
})

context("test-multiple-packages")

library(pkgload)
library(testthat)

# Setup ------------------------------------------------------------------------

# Load package and register methods
load_all("testMultiMethod")
load_all("testMultiPackage")

# Unload when finished testing
teardown({
  unload("testMultiMethod")
  unload("testMultiPackage")
})

# Testing ----------------------------------------------------------------------

test_that("multiple packages have multiple headers", {
  expect_known_cat(
    methods_rd("multi_method"),
    "testMultiPackage/test-1.txt"
  )
})

context("test-special-cases")

test_that("no methods case returns default output", {
  expect_equal(
    methods_rd("methods_rd"),
    "No methods found in currently loaded packages."
  )
})

test_that("nonexistant generic is an error", {
  expect_error(methods_rd("xyz"))
})

context("test-generics-extension")

library(pkgload)
library(testthat)

# Setup ------------------------------------------------------------------------

# Load package and register methods
load_all("testGenericsExtension")

# Unload when finished testing
teardown({
  unload("testGenericsExtension")
})

# Testing ----------------------------------------------------------------------

test_that("generics methods can be reexported and extended", {
  expect_known_cat(
    methods_rd("tidy"),
    "testGenericsExtension/test-1.txt"
  )
})

#' Set operations
#'
#' Union (`union()`), intersect (`intersect()`), difference (`setdiff()`),
#' and equality (`setequal()`) for two vectors representing sets. Determine
#' membership with `is.element()`.
#'
#' These functions override the set functions provided in base to make them
#' generic so that packages can provide methods for different data types. The
#' default methods call the base versions.
#'
#' @section Methods:
#'
#' \subsection{`intersect()`}{
#'     \Sexpr[stage=render,results=rd]{generics:::methods_rd("intersect")}
#' }
#'
#' \subsection{`union()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("union")}
#' }
#'
#' \subsection{`setdiff()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("setdiff")}
#' }
#'
#' \subsection{`setequal()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("setequal")}
#' }
#'
#' \subsection{`is.element()`}{
#'    \Sexpr[stage=render,results=rd]{generics:::methods_rd("is.element")}
#' }
#'
#' @param x,y Vectors to combine.
#' @param el,set Element and set to compare.
#' @param ... Other arguments passed on to methods.
#' @return For `union()`, `intersect()`, and `setdiff()`, a vector with all
#'   duplicate removed.
#'
#'   For `setequal()` and `is.element()`,  a logical `TRUE` or `FALSE`.`
#' @name setops
#' @examples
#' intersect(1:5, 4:8)
#' union(1:5, 4:8)
#'
#' setdiff(1:5, 4:8)
#' setdiff(4:8, 1:5)
NULL

#' @rdname setops
#' @export
intersect <- function(x, y, ...) UseMethod("intersect")
#' @rdname setops
#' @export
union <- function(x, y, ...) UseMethod("union")
#' @rdname setops
#' @export
setdiff <- function(x, y, ...) UseMethod("setdiff")
#' @rdname setops
#' @export
setequal <- function(x, y, ...) UseMethod("setequal")
#' @rdname setops
#' @export
is.element <- function(el, set, ...) UseMethod("is.element")

#' @export
intersect.default <- function(x, y, ...) base::intersect(x, y, ...)
#' @export
union.default <- function(x, y, ...) base::union(x, y, ...)
#' @export
setdiff.default <- function(x, y, ...) base::setdiff(x, y, ...)
#' @export
setequal.default <- function(x, y, ...) base::setequal(x, y, ...)
#' @export
is.element.default <- function(el, set, ...) base::is.element(el, set, ...)

#' Accuracy measures for a model
#'
#' Returns range of summary measures of the forecast accuracy.
#'
#' @param object A model for which forecasts are required.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("accuracy")}
#'
#' @export
accuracy <- function(object, ...) {
  UseMethod("accuracy")
}

#' Estimate model parameters.
#'
#' Estimates parameters for a given model from a set of data in the form of
#'  a set of predictors (`x`) and outcome(s) (`y`).
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("fit_xy")}
#'
#' @inheritParams compile
#' @export
fit_xy <- function(object, ...) {
  UseMethod("fit_xy")
}

#' Forecasting from an object
#'
#' The functions allow producing forecasts based on the provided object.
#'
#' @param object A model for which forecasts are required.
#' @param ... Other arguments passed to methods
#'
#' @section Methods:
#' \Sexpr[stage=render,results=rd]{generics:::methods_rd("forecast")}
#'
#' @export
forecast <- function(object, ...) {
  UseMethod("forecast")
}

# Generics

[![azure-devops-builds](https://img.shields.io/azure-devops/build/proofit404/generics/4?style=flat-square)](https://dev.azure.com/proofit404/generics/_build/latest?definitionId=4&branchName=master)
[![azure-devops-coverage](https://img.shields.io/azure-devops/coverage/proofit404/generics/4?style=flat-square)](https://dev.azure.com/proofit404/generics/_build/latest?definitionId=4&branchName=master)
[![pypi](https://img.shields.io/pypi/v/generics?style=flat-square)](https://pypi.python.org/pypi/generics/)
[![python](https://img.shields.io/pypi/pyversions/generics?style=flat-square)](https://pypi.python.org/pypi/generics/)

A classy toolkit designed with OOP in mind.

**[Documentation](https://proofit404.github.io/generics/) |
[Source Code](https://github.com/proofit404/generics) |
[Task Tracker](https://github.com/proofit404/generics/issues)**

In our opinion, main benefits of having objects implemented in the language are
encapsulation and polymorphous. Classes that could be easily used in a
composition are tricky to write. The `generics` library aims to help you in
writing code with high quality.

## Pros

- Real private attributes without loosing the readability
- Leads to a better design forcing you to use encapsulation properly
- Makes writing quality code with high cohesion and low coupling easier
- Guides you to follow SOLID principles

## Example

The `generics` library gives you an easy way to define private attributes on
objects without loosing little nice things like readability.

```pycon

>>> from generics import private

>>> @private
... class User:
...     def __init__(self, name):
...         self.name = name
...
...     def greet(self):
...         return f'Hello, {self.name}'

>>> user = User('Jeff')

>>> user.greet()
'Hello, Jeff'

>>> hasattr(user, 'name')
False

```

## Questions

If you have any questions, feel free to create an issue in our
[Task Tracker](https://github.com/proofit404/generics/issues). We have the
[question label](https://github.com/proofit404/generics/issues?q=is%3Aopen+is%3Aissue+label%3Aquestion)
exactly for this purpose.

## Enterprise support

If you have an issue with any version of the library, you can apply for a paid
enterprise support contract. This will guarantee you that no breaking changes
will happen to you. No matter how old version you're using at the moment. All
necessary features and bug fixes will be backported in a way that serves your
needs.

Please contact [proofit404@gmail.com](mailto:proofit404@gmail.com) if you're
interested in it.

## License

Generics library is offered under the two clause BSD license.

<p align="center">&mdash; ⭐️ &mdash;</p>
<p align="center"><i>The generics library is part of the SOLID python family.</i></p>

# generics methods can be reexported and extended

    Code
      methods_rd("tidy")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testGenericsExtension}\n\\itemize{\n\\item \\code{\\link[testGenericsExtension]{tidy-special}}: \\code{special_method}\n}"

# multiple packages have multiple headers

    Code
      methods_rd("multi_method")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testMultiMethod}\n\\itemize{\n\\item \\code{\\link[testMultiMethod]{multi-method-2}}: \\code{default}\n\\item \\code{\\link[testMultiMethod]{multi-method-3}}: \\code{data.frame}\n}\n\\code{testMultiPackage}\n\\itemize{\n\\item \\code{\\link[testMultiPackage]{multi-method-4}}: \\code{matrix}\n}"

# S4 bullets print with no issues

    Code
      methods_rd("multi_method")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testS4Docs}\n\\itemize{\n\\item \\code{\\link[testS4Docs]{multi_method}}: \\code{ANY,ANY}, \\code{numeric,ANY}, \\code{numeric,integer}\n}"

# S4 and S3 packages can intermingle

    Code
      methods_rd("multi_method")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testMultiMethod}\n\\itemize{\n\\item \\code{\\link[testMultiMethod]{multi-method-2}}: \\code{default}\n\\item \\code{\\link[testMultiMethod]{multi-method-3}}: \\code{data.frame}\n}\n\\code{testS4Docs}\n\\itemize{\n\\item \\code{\\link[testS4Docs]{multi_method}}: \\code{ANY,ANY}, \\code{numeric,ANY}, \\code{numeric,integer}\n}"

# multiple methods but same rdname are comma separated

    Code
      methods_rd("same_rd_name")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testSameRd}\n\\itemize{\n\\item \\code{\\link[testSameRd]{same_rd_name-2}}: \\code{data.frame}, \\code{default}\n}"

# single method is correctly itemized

    Code
      methods_rd("single_method")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testSingleMethod}\n\\itemize{\n\\item \\code{\\link[testSingleMethod]{single-method-2}}: \\code{default}\n}"

# multiple methods are correctly itemized

    Code
      methods_rd("multi_method")
    Output
      [1] "See the following help topics for more details about individual methods:\n\n\\code{testMultiMethod}\n\\itemize{\n\\item \\code{\\link[testMultiMethod]{multi-method-2}}: \\code{default}\n\\item \\code{\\link[testMultiMethod]{multi-method-3}}: \\code{data.frame}\n}"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{visualize}
\alias{visualize}
\title{Visualize a data set or object.}
\usage{
visualize(x, ...)
}
\arguments{
\item{x}{A data frame or other object.}

\item{...}{Other arguments passed to methods}
}
\description{
Visualize a data set or object.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("visualize")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equation.R
\name{equation}
\alias{equation}
\title{Model equations}
\usage{
equation(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Other arguments passed to methods}
}
\value{
Markup output suitable for rendering the equation.
}
\description{
Display the mathematical representation of a fitted model.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("equation")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explain.R
\name{explain}
\alias{explain}
\title{Explain details of an object}
\usage{
explain(x, ...)
}
\arguments{
\item{x}{An object. See the individual method for specifics.}

\item{...}{other arguments passed to methods}
}
\description{
Explain details of an object
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("explain")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{single_method.default}
\alias{single_method.default}
\title{Test default}
\usage{
\method{single_method}{default}(x)
}
\arguments{
\item{x}{A parameter}
}
\description{
Test default
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics-package.R
\docType{package}
\name{generics-package}
\alias{generics}
\alias{generics-package}
\title{generics: common S3 generics}
\description{
These are generic functions that can be used to minimize
package dependencies when multiple packages have the same
method.
}
\details{
An attempt was made to use generic class signatures that were
consistent with existing code. For example, \pkg{dplyr} and
\pkg{lime} both have \code{explain()} methods that do very different
things but both take \code{x} as their main object. Even though \code{x}
would be different in those cases, this package provides access
to the S3 generic so that other packages that may want to create
\code{explain()} methods for their objects can do so without loading
either of the other packages.

For example, if a new \code{tidy()} method is being developed for a
package, this lightweight package can be the required dependency
to have access to the generic method (instead of depending on
\pkg{broom} and installing its dependencies).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://generics.r-lib.org}
  \item \url{https://github.com/r-lib/generics}
  \item Report bugs at \url{https://github.com/r-lib/generics/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@rstudio.com}

Authors:
\itemize{
  \item Max Kuhn \email{max@rstudio.com}
  \item Davis Vaughan \email{davis@rstudio.com}
}

Other contributors:
\itemize{
  \item RStudio [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{multi_method}
\alias{multi_method}
\title{Generic test for multiple methods}
\usage{
multi_method(x, y)
}
\arguments{
\item{x, y}{A parameter}
}
\description{
Generic test for multiple methods
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank_results.R
\name{rank_results}
\alias{rank_results}
\title{Compute relative rankings of a collection of objects}
\usage{
rank_results(x, ...)
}
\arguments{
\item{x}{A collection of objects}

\item{...}{Other arguments passed to methods}
}
\description{
\code{rank_results()} computes relative ranks of a collection of objects and
returns a summary of the results.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("rank_results")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_imp.R
\name{var_imp}
\alias{var_imp}
\title{Calculation of variable importance}
\usage{
var_imp(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Other arguments passed to methods}
}
\description{
A generic method for calculating variable importance for model objects.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("var_imp")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\docType{methods}
\name{multi_method}
\alias{multi_method}
\alias{multi_method,ANY,ANY-method}
\alias{multi_method,numeric,ANY-method}
\alias{multi_method,numeric,integer-method}
\title{An s4 generic}
\usage{
multi_method(x, y)

\S4method{multi_method}{numeric,ANY}(x, y)

\S4method{multi_method}{numeric,integer}(x, y)
}
\description{
An s4 generic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compile.R
\name{compile}
\alias{compile}
\title{Configure an object}
\usage{
compile(object, ...)
}
\arguments{
\item{object}{An object. See the individual method for specifics.}

\item{...}{Other arguments passed to methods}
}
\description{
Finalizes or completes an object.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("compile")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prune.R
\name{prune}
\alias{prune}
\title{Prune or reduce an object}
\usage{
prune(tree, ...)
}
\arguments{
\item{tree}{A fitted model object.}

\item{...}{Other arguments passed to methods}
}
\description{
Prune or reduce an object
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("prune")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tunable.R
\name{tunable}
\alias{tunable}
\title{Declare tunable parameters}
\usage{
tunable(x, ...)
}
\arguments{
\item{x}{An object, such as a recipe, recipe step, workflow, or model
specification.}

\item{...}{Other arguments passed to methods}
}
\value{
A tibble with a column for the parameter \code{name}, information on the
\emph{default} method for generating a corresponding parameter object, the
\code{source} of the parameter (e.g. "recipe", etc.), and the \code{component} within
the source. For the \code{component} column, a little more specificity is given
about the location of the parameter (e.g. "step_normalize" or recipes or
"boost_tree" for models). The \code{component_id} column contains the unique step
\code{id} field or, for models, a logical for whether the model specification
argument was a main parameter or one associated with the engine.
}
\description{
Returns information on potential hyper-parameters that can be optimized.
}
\details{
For a model specification, an engine must be chosen.

If the object has no tunable parameters, a tibble with no rows is returned.

The information about the default parameter object takes the form of a
named list with an element for the function call and an optional element for
the source of the function (e.g. the \code{dials} package). For model
specifications, If the parameter is unknown to the underlying \code{tunable}
method, a \code{NULL} is returned.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("tunable")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explore.R
\name{explore}
\alias{explore}
\title{Create an interactive visualization appropriate to a particular object type}
\usage{
explore(x, ...)
}
\arguments{
\item{x}{A object}

\item{...}{Other arguments passed to methods}
}
\value{
\code{NULL} (invisibly) or some other data type (e.g. tibble) depending on
the application.
}
\description{
\code{explore()} invokes a function that starts an interactive, pre-defined widget
(e.g. \code{plotly} visualization, \code{shiny} app, etc.) to investigate the results.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("explore")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varying_args.R
\name{varying_args}
\alias{varying_args}
\title{Find any arguments that are not fully specified.}
\usage{
varying_args(object, ...)
}
\arguments{
\item{object}{An object. See the individual method for specifics.}

\item{...}{Other arguments passed to methods}
}
\description{
Find any arguments that are not fully specified.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("varying_args")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{same_rd_name}
\alias{same_rd_name}
\title{Generic test for multiple methods on the same rd file}
\usage{
same_rd_name(x)
}
\arguments{
\item{x}{A parameter}
}
\description{
Should be comma separated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tune_args.R
\name{tune_args}
\alias{tune_args}
\title{Determine arguments tagged for tuning}
\usage{
tune_args(object, ...)
}
\arguments{
\item{object}{A \code{model_spec}, \code{recipe}, \code{workflow}, or other object.}

\item{...}{Other arguments passed to methods.}
}
\value{
A tibble with columns for the parameter name (\code{name}), whether it
contains \emph{any} tunable value (\code{tune}), the \code{id} for the parameter (\code{id}),
and the information on where the parameter was located (\code{source}).
}
\description{
\code{tune_args()} takes an object such as a model specification or a recipe and
returns a tibble of information on all possible tunable arguments and
whether or not they are actually tunable.
}
\details{
The \code{source} column is determined differently for a
\code{model_spec} or a \code{recipe} (with additional detail on the type).

The \code{id} field has any identifier that was passed from \code{tune::tune()} (e.g.
\code{tune("some note")}). If no additional detail was used in that function,
the \code{id} field reverts to the name of the parameters.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("tune_args")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coercion.R
\name{coercion-time-difference}
\alias{coercion-time-difference}
\alias{as.difftime}
\alias{as.difftime.default}
\title{Time difference coercion}
\usage{
as.difftime(tim, ...)

\method{as.difftime}{default}(tim, format = "\%X", units = "auto", ...)
}
\arguments{
\item{tim}{A vector specifying a time interval.}

\item{...}{Other arguments passed on to methods.}

\item{format}{A single character specifying the format of \code{tim} when it is
a character. The default is a locale-specific time format.}

\item{units}{A single character specifying units in which the results are
desired. Required if \code{tim} is a numeric.}
}
\value{
A \code{difftime} object with an attribute indicating the units.
}
\description{
Coercion functions for creating \code{difftime} objects from other
existing objects.
}
\details{
This function overrides the non-generic \code{as.difftime()} function
provided in base so that packages can provide methods for different data
types. The default method call the base version.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("as.difftime")}
}

\examples{
as.difftime(1:5, units = "secs")

as.difftime(c("01:55:22", "01:55:25"))

as.difftime("01", format = "\%H")
as.difftime("01", format = "\%H", units = "secs")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluate.R
\name{evaluate}
\alias{evaluate}
\title{Evaluate an object.}
\usage{
evaluate(x, ...)
}
\arguments{
\item{x}{An object. See the individual method for specifics.}

\item{...}{other arguments passed to methods}
}
\description{
Evaluate an object.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("evaluate")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{tidy.special_method}
\alias{tidy.special_method}
\title{Test default}
\usage{
\method{tidy}{special_method}(x, ...)
}
\arguments{
\item{x}{A parameter}
}
\description{
Test default
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_xy.R
\name{fit_xy}
\alias{fit_xy}
\title{Estimate model parameters.}
\usage{
fit_xy(object, ...)
}
\arguments{
\item{object}{An object. See the individual method for specifics.}

\item{...}{Other arguments passed to methods}
}
\description{
Estimates parameters for a given model from a set of data in the form of
a set of predictors (\code{x}) and outcome(s) (\code{y}).
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("fit_xy")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coercion.R
\name{coercion-factor}
\alias{coercion-factor}
\alias{as.factor}
\alias{as.ordered}
\title{Factor coercion}
\usage{
as.factor(x, ...)

as.ordered(x, ...)
}
\arguments{
\item{x}{A vector of data.}

\item{...}{Other arguments passed on to methods.}
}
\value{
For \code{as.factor()}, a factor. For \code{as.ordered()},
an ordered factor.
}
\description{
Coercion functions for creating factors from other existing objects.
}
\details{
These functions override non-generic factor coercion functions provided
in base so that packages can provide methods for different data types. The
default methods call the base versions.
}
\section{Methods}{


\subsection{\code{as.factor()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("as.factor")}
}

\subsection{\code{as.ordered()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("as.ordered")}
}
}

\examples{
as.factor(letters[1:5])
as.ordered(letters[1:5])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{same_rd_name.default}
\alias{same_rd_name.default}
\alias{same_rd_name.data.frame}
\title{Test default}
\usage{
\method{same_rd_name}{default}(x)

\method{same_rd_name}{data.frame}(x)
}
\arguments{
\item{x}{A parameter}

\item{x}{A parameter}
}
\description{
Test default

Test data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\docType{import}
\name{reexports}
\alias{reexports}
\alias{tidy}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{generics}{\code{\link[generics]{tidy}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sets.R
\name{setops}
\alias{setops}
\alias{intersect}
\alias{union}
\alias{setdiff}
\alias{setequal}
\alias{is.element}
\title{Set operations}
\usage{
intersect(x, y, ...)

union(x, y, ...)

setdiff(x, y, ...)

setequal(x, y, ...)

is.element(el, set, ...)
}
\arguments{
\item{x, y}{Vectors to combine.}

\item{...}{Other arguments passed on to methods.}

\item{el, set}{Element and set to compare.}
}
\value{
For \code{union()}, \code{intersect()}, and \code{setdiff()}, a vector with all
duplicate removed.

For \code{setequal()} and \code{is.element()},  a logical \code{TRUE} or \code{FALSE}.`
}
\description{
Union (\code{union()}), intersect (\code{intersect()}), difference (\code{setdiff()}),
and equality (\code{setequal()}) for two vectors representing sets. Determine
membership with \code{is.element()}.
}
\details{
These functions override the set functions provided in base to make them
generic so that packages can provide methods for different data types. The
default methods call the base versions.
}
\section{Methods}{


\subsection{\code{intersect()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("intersect")}
}

\subsection{\code{union()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("union")}
}

\subsection{\code{setdiff()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("setdiff")}
}

\subsection{\code{setequal()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("setequal")}
}

\subsection{\code{is.element()}}{
\Sexpr[stage=render,results=rd]{generics:::methods_rd("is.element")}
}
}

\examples{
intersect(1:5, 4:8)
union(1:5, 4:8)

setdiff(1:5, 4:8)
setdiff(4:8, 1:5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{single_method}
\alias{single_method}
\title{Generic test}
\usage{
single_method(x)
}
\arguments{
\item{x}{A parameter}
}
\description{
Generic test
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{components}
\alias{components}
\title{Extract components}
\usage{
components(object, ...)
}
\arguments{
\item{object}{A data separable object.}

\item{...}{Other arguments passed to methods}
}
\value{
A dataset (\code{\link[tibble:tibble]{tibble::tibble()}} or similar) containing components from
the object.
}
\description{
\code{components} can be used to extract elements from an object.
}
\details{
For example, decomposition methods and some modelling techniques can be used
to decompose a dataset into components of interest. This function is used
to extract these components in a tidy data format.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("components")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{multi_method.matrix}
\alias{multi_method.matrix}
\title{Test data frame}
\usage{
multi_method.matrix(x, y)
}
\arguments{
\item{x, y}{A parameter}
}
\description{
Test data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/min_grid.R
\name{min_grid}
\alias{min_grid}
\title{Determine the minimum set of model fits}
\usage{
min_grid(x, grid, ...)
}
\arguments{
\item{x}{A model specification.}

\item{grid}{A tibble with tuning parameter combinations.}

\item{...}{Not currently used.}
}
\value{
A tibble with the minimum tuning parameters to fit and an additional
list column with the parameter combinations used for prediction.
}
\description{
\code{min_grid()} determines exactly what models should be fit in order to
evaluate the entire set of tuning parameter combinations. This is for
internal use only and the API may change in the near future.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("min_grid")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a.r
\name{multi_method.default}
\alias{multi_method.default}
\title{Test default}
\usage{
\method{multi_method}{default}(x, y)
}
\arguments{
\item{x, y}{A parameter}
}
\description{
Test default
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/refit.R
\name{refit}
\alias{refit}
\title{Refitting models}
\usage{
refit(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Other arguments passed to methods}
}
\description{
Refitting models
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("refit")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate.R
\name{generate}
\alias{generate}
\title{Generate values based on inputs}
\usage{
generate(x, ...)
}
\arguments{
\item{x}{An object.}

\item{...}{Other arguments passed to methods}
}
\description{
Generate values based on inputs
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("generate")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/required_pkgs.R
\name{required_pkgs}
\alias{required_pkgs}
\title{Determine packages required by objects}
\usage{
required_pkgs(x, ...)
}
\arguments{
\item{x}{An object.}

\item{...}{Other arguments passed to methods}
}
\value{
A character string of packages that are required.
}
\description{
Determine packages required by objects
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("required_pkgs")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glance.R
\name{glance}
\alias{glance}
\title{Glance at an object}
\usage{
glance(x, ...)
}
\arguments{
\item{x}{model or other R object to convert to single-row data frame}

\item{...}{other arguments passed to methods}
}
\description{
Construct a single row summary "glance" of a model, fit, or other
object
}
\details{
glance methods always return either a one-row data frame (except on
\code{NULL}, which returns an empty data frame)
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("glance")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/train.R
\name{train}
\alias{train}
\title{Estimate model parameters.}
\usage{
train(x, ...)
}
\arguments{
\item{x}{An object. See the individual method for specifics.}

\item{...}{other arguments passed to methods}
}
\description{
Estimates parameters for a given model from a set of data.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("train")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{tidy}
\alias{tidy}
\title{Turn an object into a tidy tibble}
\usage{
tidy(x, ...)
}
\arguments{
\item{x}{An object to be converted into a tidy \code{\link[tibble:tibble]{tibble::tibble()}}.}

\item{...}{Additional arguments to tidying method.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with information about model components.
}
\description{
Turn an object into a tidy tibble
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("tidy")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast}
\alias{forecast}
\title{Forecasting from an object}
\usage{
forecast(object, ...)
}
\arguments{
\item{object}{A model for which forecasts are required.}

\item{...}{Other arguments passed to methods}
}
\description{
The functions allow producing forecasts based on the provided object.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("forecast")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpolate.R
\name{interpolate}
\alias{interpolate}
\title{Interpolate missing values}
\usage{
interpolate(object, ...)
}
\arguments{
\item{object}{A fitted model object}

\item{...}{Other arguments passed to methods}
}
\value{
A dataset (\code{\link[tibble:tibble]{tibble::tibble()}} or similar) of the same structure as
the input dataset with missing values from the response variable replaced
with interpolated values.
}
\description{
Interpolates missing values provided in the training dataset using the
fitted model.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("interpolate")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{augment}
\alias{augment}
\title{Augment data with information from an object}
\usage{
augment(x, ...)
}
\arguments{
\item{x}{Model object or other R object with information to append to
observations.}

\item{...}{Addition arguments to \code{augment} method.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with information about data points.
}
\description{
Augment data with information from an object
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("augment")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate.R
\name{calculate}
\alias{calculate}
\title{Calculate statistics.}
\usage{
calculate(x, ...)
}
\arguments{
\item{x}{An object.}

\item{...}{Other arguments passed to methods}
}
\description{
Calculate statistics.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("calculate")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estfun.R
\name{estfun}
\alias{estfun}
\title{Extracting the estimating functions of a fitted model.}
\usage{
estfun(x, ...)
}
\arguments{
\item{x}{A fitted model object.}

\item{...}{Other arguments passed to methods}
}
\description{
Extracting the estimating functions of a fitted model.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("estfun")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accuracy.R
\name{accuracy}
\alias{accuracy}
\title{Accuracy measures for a model}
\usage{
accuracy(object, ...)
}
\arguments{
\item{object}{A model for which forecasts are required.}

\item{...}{Other arguments passed to methods}
}
\description{
Returns range of summary measures of the forecast accuracy.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("accuracy")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/specify.R
\name{specify}
\alias{specify}
\title{Specify variables or other quantities.}
\usage{
specify(x, ...)
}
\arguments{
\item{x}{An object.}

\item{...}{Other arguments passed to methods}
}
\description{
Specify variables or other quantities.
}
\section{Methods}{

\Sexpr[stage=render,results=rd]{generics:::methods_rd("specify")}
}
